/* Tiny2.m
 *
 * A tiny Cocoa application which creates a 
 * a window and displays graphics in it.
 * 
 * (C) 2002 by Simson L. Garfinkel and Michael K. Mahoney.
 * all rights reserved.
 */

#import <Cocoa/Cocoa.h>

/****************************************************************
 ** The CView draws the image.
 ****************************************************************/

@interface CView : NSView
{
}
- (void)drawRect:(NSRect)rect;
- (void)mouseDown:(NSEvent *)theEvent;
- (void)windowWillClose:(NSNotification *)notification;
@end

@implementation CView

- (void)drawRect:(NSRect)rect
{
    /* Clear the background */

    [[NSColor whiteColor] set];
    NSRectFill([self bounds]);
}

-(void)windowWillClose:(NSNotification *)notification
{
    [NSApp terminate:self];
}

- (void)mouseDown:(NSEvent *)theEvent
{
    NSRect r;

    r.origin = [theEvent locationInWindow];
    r.origin.x -= 5;
    r.origin.y -= 5;
    r.size.width = 10;
    r.size.height = 10;

    [[NSColor blackColor] set];
    [self lockFocus];
    [NSBezierPath fillRect:r];
    [self unlockFocus];
    [[self window] flushWindow];

}

@end

void setup()
{
    NSWindow *myWindow;
    NSView   *theView;
    NSRect   graphicsRect;

    /* Now create the window */

    graphicsRect = NSMakeRect(100.0, 350.0, 400.0, 400.0);
    myWindow = [ [NSWindow alloc] 
	       initWithContentRect: graphicsRect
	       styleMask:   NSTitledWindowMask
	                    |NSClosableWindowMask
	                    |NSMiniaturizableWindowMask 
	       backing:     NSBackingStoreBuffered
	       defer:       NO];

    [myWindow setTitle:@"Tiny Application Window"];

    /* Create the CView for the Window.  */
    theView = [ [CView alloc] init];
    [myWindow setContentView:theView ];
    [myWindow setDelegate:theView ];
    [myWindow makeKeyAndOrderFront: nil];
}    

int main()
{
    /* Create the autorelease pool           */
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    /* Create the application object.        */
    NSApp = [[NSApplication alloc] init];

    setup();

    /* run the main event loop */
    [NSApp run];							

    /* We get here when the user closes the window. */

    [NSApp release];				  /* free the app */
    [pool release];				  /* release the autorelease pool */
    return(EXIT_SUCCESS);
}
