#import "PaperController.h"

@implementation PaperController

- (void)windowDidLoad
{
    NSString *path=0;

    [super windowDidLoad];  
    [ [self window] makeFirstResponder:theText]; 

    path = [ [NSBundle mainBundle]
	       pathForResource:@"Evaluator" ofType:@""];

    toPipe   = [NSPipe pipe];
    fromPipe = [NSPipe pipe];

    toEvaluator   = [toPipe fileHandleForWriting];
    fromEvaluator = [fromPipe fileHandleForReading];

    evaluator = [ [NSTask alloc] init];
    [evaluator setLaunchPath:path];

    [evaluator setStandardOutput:fromPipe];
    [evaluator setStandardInput:toPipe];
    [evaluator launch];

    [ [NSNotificationCenter defaultCenter]
	addObserver:self
	selector:@selector(gotData:)
	name:NSFileHandleReadCompletionNotification
	object:fromEvaluator];

    [fromEvaluator readInBackgroundAndNotify];
}


- (void)dealloc
{
    [ [NSNotificationCenter defaultCenter] removeObserver:self];
    [evaluator terminate];
    [evaluator release];
    [super dealloc];
}



// NSTextView delegate method -textDidChange:
// If current event is a carriage return do special processing

- (void)textDidChange:(NSNotification *)notification
{
    NSString *str = [ [ [self window] currentEvent] characters];

    if ([str isEqualToString:@"\r"]) {
	// Get the last line of text and send it to the Evaluator
	NSString *str = [theText string];

	int i;
	for (i=[str length]-2;i>=0;i--) {
	    if (i==0 || [str characterAtIndex:i-1] == '\n') {
		NSRange llRange = NSMakeRange(i,[str length]-i);
		NSString *lastLine =  [str substringWithRange:llRange];

		[toEvaluator writeData:
				 [lastLine dataUsingEncoding:NSASCIIStringEncoding
					   allowLossyConversion:YES] ];

		// Do not allow any more changes to the text
		[theText setEditable:NO];

		return;
	    }
	}
    }
}


- (void)gotData:(NSNotification *)notification
{
    NSData *data;
    NSString *str;

    data = [ [notification userInfo]
	       objectForKey:NSFileHandleNotificationDataItem];

    str = [ [NSString alloc] initWithData:data
			     encoding:NSASCIIStringEncoding] ;

    // Add the data to the end of theText object.
    [theText appendString:str];
    [theText appendString:@"--------------------\n"];

    // Scroll to the bottom.
    [theText scrollRangeToVisible:
		 NSMakeRange([ [theText textStorage] length], 0)];

    // Register to get the notification again
    [fromEvaluator readInBackgroundAndNotify];

    // Allow the user to type additional math expressions.
    [theText setEditable:YES];
    [str release];
}

- (NSString *)windowTitleForDocumentDisplayName:
    (NSString *)displayName
{
    return [@"MathPaper: " stringByAppendingString:displayName];

}



@end



@implementation NSTextView(MathPaper)
- (void)appendString:(NSString *)str
{
    int len = [ [self textStorage] length];
    [self replaceCharactersInRange:NSMakeRange(len,0)
	  withString:str];
}
@end
