// MathDocument.m

#import "MathDocument.h"
#import "PaperController.h"
#import "RTF.h"

@implementation MathDocument

- (NSString *)windowNibName
{
    // Override returning the nib file name of the document
    // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    return @"PaperWindow";
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    [super windowControllerDidLoadNib:aController];
    // Add any code here that need to be executed once the windowController has loaded the document's window.
}

- (NSData *)dataRepresentationOfType:(NSString *)aType
{
    // Insert code here to write your document from the given data.  You can also choose to override -fileWrapperRepresentationOfType: or -writeToFile:ofType: instead.

    if([aType isEqualToString:@"MathPaper"]) {

        /* Ask our windows to synchronize their data */
        [[self windowControllers]
            makeObjectsPerformSelector:@selector(synchronizeData)];

        /* And encode the data */
        return [NSArchiver archivedDataWithRootObject:self];
    }
    return nil;                         // cannot encode
}


- (BOOL)loadDataRepresentation:(NSData *)newHistory
        ofType:(NSString *)aType
{
    if([aType isEqualToString:@"MathPaper"]){
        MathDocument *temp;

        temp = [NSUnarchiver unarchiveObjectWithData:newHistory];
        if(temp){
            [self setFrame:[temp getFrame]];
            [self setHistory:[temp getHistory]];
            return YES;
        }
    }
    return NO;
}

- (id)initWithCoder:(NSCoder *)coder
{
    frame   = [coder decodeRect];
    history = [[coder decodeObject] retain];
    return self;
}




// Override the NSDocument makeWindowControllers
// method to specify our own controller.

- (void)makeWindowControllers
{
   id ctl = [ [PaperController alloc]
               initWithWindowNibName:[self windowNibName] ];

   [ctl  autorelease];
   [self addWindowController:ctl];
}

//======================================


-(NSData *)getHistory
{
    return history;
}

-(void)setHistory:(NSData *)theHistory
{
    [history setData:theHistory];
}

-(NSRect)getFrame
{
    return frame;
}

-(void)setFrame:(NSRect)aFrame
{
    frame = aFrame;
}

-(BOOL)hasFrame
{
    return frame.size.height!=0 && frame.size.width!=0;
}


//======================================


// Designated initializer; create the empty document
-(id)init
{
    RTF *rtf = [ [ [RTF alloc] init] autorelease];

    [super init];
    [rtf setBold:TRUE];
    [rtf setSize:10.0];
    [rtf appendString:@"Enter a math expression and hit return:"];
    [rtf setSize:12.0];
    [rtf appendString:@"\n"];
    history = [ [NSMutableData alloc] initWithData:[rtf data] ];
    return self;
}

-(void)dealloc
{
    [history release];
    [super dealloc];
}


- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeRect:frame];
    [coder encodeObject:history];
}



- (BOOL)writeToFile:(NSString *)fileName ofType:(NSString *)type
{
    BOOL ret  = [super writeToFile:fileName ofType:type];
    if (ret) {
        PaperController *pctl =  [ [self windowControllers] objectAtIndex:0];
        [ [pctl window] setDocumentEdited:NO];
    }
    return ret;
}



- (BOOL)isDocumentEdited
{
    PaperController *pctl =
        [ [self windowControllers] objectAtIndex:0];
    return [ [pctl window] isDocumentEdited];
}




@end

