#import "PolygonView.h"


@implementation PolygonView

- (IBAction)takeNumSidesFrom:sender
{
	[self setNumSides:[sender intValue] ];
}


- initWithFrame:(NSRect)rect
{
    [super initWithFrame:rect];
    [self setBounds:NSMakeRect(-1,-1,2,2)];
    [self setNumSides:3];
    
    colors = [[NSMutableArray alloc] init];

    [colors addObject:[NSColor blackColor]];
    [colors addObject:[NSColor blueColor]];
    [colors addObject:[NSColor redColor]];
    [colors addObject:[NSColor greenColor]];
    [colors addObject:[NSColor whiteColor]];
    
    return self;
}


- (void)dealloc
{
    [colors release];
    [super dealloc];
}

- (BOOL)isOpaque
{
    return NO;
}


- (void)setNumSides:(int)val
{
    if (val>0 && sides!=val) {
        sides = val;
        [self setNeedsDisplay:YES];
    }
}


-(void)drawRect:(NSRect)rect
{
    float theta;

    if(shape) {  // new
        [shape release];
        shape = 0;
    }

    shape = [ [NSBezierPath bezierPath] retain];  // new

    [shape moveToPoint:NSMakePoint(sin(0.0),cos(0.0))];

    // M_PI is a predefined value of PI.
    // M_PI*2.0 is number of radians in a circle.
    // The for() statement below sweeps through each
    //    pie-section of the polygon for each side.

    for (theta=0.0;
         theta <= 2*M_PI;
         theta += (M_PI*2.0)/sides) {

        [shape lineToPoint:NSMakePoint(sin(theta),cos(theta)) ];
    }
    [ [colors objectAtIndex:colorNum] set];  // new
    [shape fill];
}



- (void)setSize:(float)size
{
    [self setFrameSize:NSMakeSize(size,size)];
    [self setBounds:NSMakeRect(-1,-1,2,2)];
    [self setNeedsDisplay:YES];
}


- (void)takeFloatSize:(id)sender
{
    [self setSize:[sender floatValue] ];
}



- (void)mouseDown:(NSEvent *)theEvent
{
    NSPoint loc = [self convertPoint:
                        [theEvent locationInWindow] fromView:nil];

    if ([shape containsPoint:loc]) {
        colorNum = (colorNum+1) % [colors count];
        [self setNeedsDisplay:YES];
    }
    else {
          NSRunAlertPanel([self description],
                              @"You missed the shape!",0,0,0);
    }
}



@end
