#import "ColorGraphView.h"
#import "Segment.h"
#import "Label.h"


@implementation ColorGraphView

- (void)dealloc
{
    [axesColor release];
    [graphColor release];
    [labelColor release];
    [super dealloc];
}

- (NSColor *)colorForTag:(int)aTag
{
    switch(aTag) {
    case AXES_TAG:      return axesColor;
    case GRAPH_TAG:     return graphColor;
    case LABEL_TAG:     return labelColor;
    }
    return nil;                         // no color?
}

- (void)setObjectsToColor:(NSColor *)theColor forTag:(int)aTag
{
    NSEnumerator *en;
    id obj=nil;

    /* First set the correct instance variable */
    switch(aTag){
    case AXES_TAG:
        [axesColor release];
        axesColor = [theColor retain];
        break;

    case GRAPH_TAG:
        [graphColor release];
        graphColor = [theColor retain];
        break;

    case LABEL_TAG:
        [labelColor release];
        labelColor = [theColor retain];
        break;
    }

    // Now set the elements in the display list
    en = [displayList objectEnumerator];
    while(obj = [en nextObject]){
        if([obj tag]==aTag){
            [obj setColor:theColor];
            [self setNeedsDisplayInRect:[obj bounds]];
        }
    }
}



- (void)addGraphElement:(id)element
{
    /* Set the color to match the request */
    [element setColor:[self colorForTag:[element tag]]];

    /* Add the element to the display list */
    [super addGraphElement:element];
}


- (IBAction)setObjectsToColor:(id)sender
{
    [self setObjectsToColor:[sender color] forTag:[sender tag]];
}


- initWithFrame:(NSRect)frame
{
    [super initWithFrame:frame];
    axesColor = [[NSColor lightGrayColor] retain];
    graphColor = [[NSColor blackColor] retain];
    labelColor = [[NSColor darkGrayColor] retain];
    return self;
}


@end
