#import "GraphView.h"
#import "Segment.h"
#import "Label.h"


@implementation GraphView

/* Note: dealloc is missing from G&M Book; does it really need to be here, 
 * as it is never called?
 */
- (void)dealloc
{
    [evaluator  release];
    [evaluator  terminate];

    [displayList release];
    [fromBuf    release];
    [super dealloc];
}

- (IBAction)graph:(id)sender
{
   // set instance variables from the form
   [self getFormAndScaleView];

   // Check the parameters of the graph
   if (xmax < xmin || ymax < ymin){
        NSRunAlertPanel( nil, @"Invalid min/max combination",
                         @"OK", nil, nil);
       return;
    }

    if ( xstep <= 0 ) {
        NSRunAlertPanel(0, @"The step size must be positive",
                        @"OK", nil, nil);
        return;
    }

    [self clear];

    // Display the axes
    [self addAxesFrom:NSMakePoint(xmin,0.0)
                   to:NSMakePoint(xmax,0.0) ];
    [self addAxesFrom:NSMakePoint(0.0,ymin)
                   to:NSMakePoint(0.0,ymax) ];
                   
                       // Add a label
    {
        Label *label = [ [Label alloc] initRect:
        NSMakeRect(xmin, ymin, xmax-xmin, (ymax-ymin)*.2)
                text:[formulaField stringValue]
                size:24.0];

	[label autorelease];
        [label setTag:LABEL_TAG];
        [self addGraphElement:label];
    }


    first = YES;
    stop_sending = NO;
    sending = YES;
    receiving = YES;

    [graphButton setTitle:@"Stop"];
    [graphButton setAction:@selector(stopGraph:)];

    [NSThread detachNewThreadSelector:@selector(sendData)
              toTarget:self
              withObject:nil];

}



- (IBAction)stopGraph:(id)sender
{
    stop_sending = YES;
}



- (void)doStop:(int)which
{
    switch (which) {
    case STOP_SENDER:
         sending = NO;
         break;
    case STOP_RECEIVER:
         receiving = NO;
         break;
    }

    if (sending==NO && receiving==NO) {  // Reinitialize
        [graphButton setTitle:@"Graph"];
        [graphButton setAction:@selector(graph:)];
        [graphButton setEnabled:YES];
    }

    if (sending==NO && receiving==YES) { // Wait for results data
        [graphButton setEnabled:FALSE];
        [graphButton setTitle:@"Waiting..."];
    }

    if (sending==YES && receiving==NO) { // A problem
        NSLog(@"Synchronization error");
    }
}



- (void)gotData:(NSNotification *)not
{
    NSData      *data;
    NSString    *str;
    NSPoint      pt;
    int          num;
    NSString    *line=0;

    data = [ [not userInfo]
             objectForKey:NSFileHandleNotificationDataItem];
    str  = [ [NSString alloc] initWithData:data
                               encoding:NSASCIIStringEncoding];

    // Add the data to the end of the text buffer
    [fromBuf appendString:str];
    [str release];

    // Register to get the notification again
    [fromEvaluator readInBackgroundAndNotify];

    // Now, process all complete lines we have
    do {
        NSRange r1;

        r1 = [fromBuf rangeOfString:@"\n"];
        if (r1.length<1) break;

        line = [fromBuf substringToIndex:r1.location];
        [fromBuf
             replaceCharactersInRange:NSMakeRange(0,r1.location+1)
                          withString:@""];

        num = sscanf( [line cString], "%f, %f", &pt.x, &pt.y);
        if (num!=2) {
            [self doStop:STOP_RECEIVER];
            return;
        }
        if (!first && !stop_sending) {
            Segment *seg = [ [Segment alloc]
                             initFrom:lastPt to:pt ];
            [seg  setTag:GRAPH_TAG];
	    [seg  autorelease];
            [self addGraphElement:seg];
        }
        first = NO;               // No longer first
        lastPt = pt;              // Remember this point
    } while (line);
    // End of data
}




- (void)sendData
{
    NSAutoreleasePool *threadPool =
                     [ [NSAutoreleasePool alloc] init];
    NSString *formula;
    double x;
    int i;

    formula = [formulaField stringValue];

    for (x=xmin; stop_sending==NO && x<=xmax; x+=xstep) {

        NSMutableString *fsend =
              [NSMutableString stringWithString:@"x,"];
        NSString *xString = [NSString stringWithFormat:@"%g",x];

        [fsend appendString:formula];
        [fsend appendString:@"\n"];

        // Now go through the formula and change
        // every 'x' to a '%g'

        for (i=[fsend length]-1; i>=0; i--) {
            if ([fsend characterAtIndex:i] == 'x') {
                [fsend replaceCharactersInRange:NSMakeRange(i,1)
                                     withString:xString];
            }
        }

        // Send this to the other side
        [toEvaluator writeData:
               [fsend dataUsingEncoding:NSASCIIStringEncoding
                   allowLossyConversion:YES] ];
    }
    // Now send through the termination code
    [toEvaluator writeData:[@"999\n"
         dataUsingEncoding:NSASCIIStringEncoding
      allowLossyConversion:YES] ];

    [self doStop:STOP_SENDER];

    // When this returns, our thread will die.
    [threadPool release];
}



- initWithFrame:(NSRect)frame
{
    NSString *path;

    [super initWithFrame:frame];

    displayList = [ [NSMutableArray alloc] init];
    fromBuf = [ [NSMutableString alloc] init];

    // What follows is largely from MathPaper
    path  = [ [NSBundle mainBundle]
               pathForResource:@"Evaluator" ofType:@""];

    if (!path) {
        NSLog(@"%@: Cannot find Evaluator",[self description]);
    }
    else {
        toPipe   = [NSPipe pipe];
        fromPipe = [NSPipe pipe];

        toEvaluator   = [toPipe fileHandleForWriting];
        fromEvaluator = [fromPipe fileHandleForReading];

        evaluator = [ [NSTask alloc] init];
        [evaluator setLaunchPath:path];

        [evaluator setStandardOutput:fromPipe];
        [evaluator setStandardInput:toPipe];
        [evaluator launch];

        [ [NSNotificationCenter defaultCenter]
            addObserver:self
               selector:@selector(gotData:)
                   name:NSFileHandleReadCompletionNotification
                 object:fromEvaluator ];


        [fromEvaluator readInBackgroundAndNotify];
    }

    // The notification below causes the getFormAndScaleView
    //    method to be invoked whenever this view is resized.
    [ [NSNotificationCenter defaultCenter]
                    addObserver:self
                       selector:@selector(getFormAndScaleView)
                           name:NSViewFrameDidChangeNotification
                         object:self];
    return self;
}


- (void)clear
{
    [displayList removeAllObjects];
    [self setNeedsDisplay:YES];
}


- (void)addGraphElement:(id)element
{
    [displayList addObject:element];
    [self setNeedsDisplayInRect:[element bounds]];
}


- (void)getFormAndScaleView
{
    xmin  = [xminCell doubleValue];
    xmax  = [xmaxCell doubleValue];
    xstep = [xstepCell doubleValue];
    ymin  = [yminCell doubleValue];
    ymax  = [ymaxCell doubleValue];
    [self setBounds:(NSMakeRect(xmin, ymin, xmax-xmin,
                                            ymax-ymin) ) ];
    [self setNeedsDisplay:YES];
}


-(BOOL)isOpaque { return YES; }   // Because GraphView is opaque

-(void)drawRect:(NSRect )rect
{
    id obj=nil;
    NSEnumerator *en;
    NSSize sz;

    [ [NSColor whiteColor] set];
    NSRectFill(rect);

    sz = [self convertSize:NSMakeSize(1,1) fromView:nil];
    [NSBezierPath setDefaultLineWidth:MAX(sz.width,sz.height)];

    en = [displayList objectEnumerator];
    while (obj = [en nextObject]) {
        if (NSIntersectsRect(rect,[obj bounds]) ) {
            [obj stroke];
        }
    }
}


- (void)addAxesFrom:(NSPoint)pt1 to:(NSPoint)pt2
{
    Segment *seg = [ [Segment alloc] initFrom:pt1 to:pt2];
    [seg  setTag:AXES_TAG];
    [seg  autorelease];
    [self addGraphElement:seg];
}


@end
