#import "Controller.h"
#import "GraphView.h"


@implementation Controller

- (GraphView *)graphView
{
        return graphView;
}

- (IBAction)showPrefs:(id)sender
{
    if (!prefController) {
        [NSBundle loadNibNamed:@"Preferences.nib" owner:self];
    }
    [ [prefController window] makeKeyAndOrderFront:sender];
}



// This implementation works around an AppKit bug in Cocoa 10.1
//   by placing the view in a different window and asking that
//   window to create the PDF for the view.

- (NSData *)PDFForView:(NSView *)aView
{
    NSRect frame         = [aView frame];
    NSView *oldSuperview = [aView superview];
    NSWindow *tempWindow;
    NSData *pdf;

    tempWindow = [[NSWindow alloc]
                     initWithContentRect:frame
                               styleMask:0
                                 backing:NSBackingStoreRetained
                                   defer:NO];

    [[tempWindow contentView] addSubview:aView];
    pdf = [tempWindow dataWithPDFInsideRect:[tempWindow frame]];
    [oldSuperview addSubview:aView];	// put back where it should be
    [tempWindow release];		// release the tempWindow
    return pdf;
}


/*  Replaced near the end of chapter 19

- (IBAction)saveDocumentTo:(id)sender
{
    NSSavePanel *pan = [NSSavePanel savePanel];

    [pan setRequiredFileType:@"pdf"];
    [pan setTitle:@"Save Graph"];

    [pan beginSheetForDirectory:nil
                           file:nil
                 modalForWindow:[graphView window]
                  modalDelegate:self
didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:)
                 contextInfo:nil];
}
*/



/*  Replaced near the end of chapter 19


- (void)savePanelDidEnd:(NSSavePanel *)sheet
             returnCode:(int)returnCode
            contextInfo:(void *)contextInfo
{
    NSData *graphPDF;

    if (returnCode==0) return;    // User did not click OK
                                  // Take no action
    graphPDF = [self PDFForView:graphView];

    if([graphPDF writeToFile:[sheet filename] atomically:NO]==NO){
        NSRunAlertPanel(nil,
                        @"Cannot save file '%@': %s",nil,nil,nil,
                        [sheet filename],strerror(errno));
    }
}
*/


- (NSData *)TIFFForView:(NSView *)aView
{
    NSImage *image = [[NSImage alloc] 
                         initWithData:[self PDFForView:aView]];

    [image autorelease];
    return [image TIFFRepresentation];
}



- (IBAction)setFormat:(id)sender
{
    [savePanel
        setRequiredFileType:
            [ [[sender selectedCell] title] lowercaseString] ];
}



- (void)savePanelDidEnd:(NSSavePanel *)sheet
             returnCode:(int)returnCode
            contextInfo:(void  *)contextInfo
{
    NSData *image=0;
    NSString *filetype = [sheet requiredFileType];

    if (returnCode==0) return;    // User did not click OK

    if ([filetype isEqualToString:@"pdf"]) {
        image = [self PDFForView:graphView];
    }
    if ([filetype isEqualToString:@"tiff"]) {
        image = [self TIFFForView:graphView];
    }
    if (image==0) {
        NSRunAlertPanel(nil,@"Unknown file type '%@'",
                        nil,nil,nil,filetype);
        return;
    }

    if ([image writeToFile:[sheet filename] atomically:NO]==NO) {
        NSRunAlertPanel(nil,
                        @"Cannot save file '%@': %s",nil,nil,nil,
                        [sheet filename],
                        strerror(errno));
    }
}





- (IBAction)saveDocumentTo:(id)sender
{
    NSString    *type = [ [ [formatMatrix selectedCell] title]
                                 lowercaseString];

    // Get a Save Panel
    savePanel  = [NSSavePanel savePanel];
    [savePanel setTitle:@"Save Graph Image"];

    // Set the initial file type
    [savePanel setRequiredFileType: type];

    // Put the format box in the Save Panel
    [savePanel setAccessoryView:formatBox];

    [formatBox retain];                 // keep a copy!

    // And run
    [savePanel beginSheetForDirectory:nil
                                 file:nil
                       modalForWindow:[graphView window]
                        modalDelegate:self
didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:)
                          contextInfo:nil];
}





@end
