#import "TrackingGraphView.h"
#import "Segment.h"


@implementation TrackingGraphView

- (id)initWithFrame:(NSRect)frame
{
    [super initWithFrame:frame];

    annotations = [[NSMutableArray alloc] init];

    trackingRect = [self addTrackingRect:[self visibleRect]
                                   owner:self
                                userData:0
                            assumeInside:NO];
    return self;
}


- (void)dealloc
{
    [trackingRect release];
    [super dealloc];
}    

- (void)getFormAndScaleView
{
    [self removeTrackingRect:trackingRect]; // remove the old
    [super getFormAndScaleView];
    trackingRect = [self addTrackingRect:[self visibleRect]
                                   owner:self
                                userData:0
                            assumeInside:NO];
}



- (void)mouseEntered:(NSEvent *)theEvent
{
    [ [self window] setAcceptsMouseMovedEvents:YES];
    [ [self window] makeFirstResponder:self];
}

- (void)mouseExited:(NSEvent *)theEvent
{
    [self removeAnnotations];
    [ [self window] setAcceptsMouseMovedEvents:NO];
}



- (void)addAnnotation:(id)obj
{
    [annotations addObject:obj];
    [self addGraphElement:obj];
}

- (void)removeAnnotations
{
    NSEnumerator *en = [annotations objectEnumerator];
    id obj;

    while(obj = [en nextObject]) {
        [self setNeedsDisplayInRect:[obj bounds] ];
    }

    [displayList removeObjectsInArray:annotations];
    [annotations removeAllObjects];
}


- (void)mouseMoved:(NSEvent *)theEvent
{
    NSPoint pt;
    NSEnumerator *en;
    id obj;

    pt = [self convertPoint:[theEvent locationInWindow]
                   fromView:nil];

    en = [displayList objectEnumerator];
    while (obj = [en nextObject]) {

        if ([obj tag]==GRAPH_TAG &&
           pt.x >= [obj bounds].origin.x &&
           pt.x <= [obj bounds].origin.x +
                   [obj bounds].size.width) {

    // Are we within 30 pixels of the line in screen coordinates?
            NSPoint ptMouse = [theEvent locationInWindow];
            NSPoint ptLine =
               [self convertPoint:[obj segmentCenter] toView: nil];

            double dist = sqrt(pow(ptMouse.x - ptLine.x,2) +
                               pow(ptMouse.y - ptLine.y,2));

            if (dist<30.0) {
                // Add two segments to annotations
                NSRect vb = [self bounds];
                NSRect ob = [obj bounds];
                id seg;

                [self removeAnnotations];  // Remove the old

                // Horizontal line intersecting cursor hot spot
                seg = [ [Segment alloc]
                        initFrom:NSMakePoint(vb.origin.x,
                                             ob.origin.y)
                        to:NSMakePoint(vb.origin.x+vb.size.width,
                                       ob.origin.y)];
                [self addAnnotation:seg];
                [seg setColor:[NSColor greenColor] ];


                // Vertical line intersecting cursor hot spot
                seg = [ [Segment alloc]
                         initFrom:NSMakePoint(ob.origin.x,
                                              vb.origin.y)
                         to:NSMakePoint(ob.origin.x,
                                      vb.origin.y+vb.size.height)];
                [self addAnnotation:seg];
                [seg setColor:[NSColor greenColor]];

                // Update positions in the x and y text fields
                [xCell setStringValue:
                          [NSString stringWithFormat:@"x: %g",
                                    [obj segmentCenter].x] ];
                [yCell setStringValue:
                          [NSString stringWithFormat:@"y: %g",
                                    [obj segmentCenter].y] ];

                [self display];    // Shouldn't be needed
                return;
            }
        }
   }

    // No segment should be highlighted
    [self removeAnnotations];
    [self display];
    [xCell setStringValue:@"x:"];
    [yCell setStringValue:@"y:"];
}




@end
